/*
 * (C) Copyright 2000
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */


/*-----------------------------------------------------------------------
 * FLASH Info: contains chip specific data, per FLASH bank
 */

#ifdef __ASSEMBLY__

#define FI_OFF_SIZE	 0
#define FI_OFF_SECT	 4
#define	FI_OFF_ID	 6
#define FI_OFF_FIRST	 8
#define FI_OFF_LAST	10
#define FI_OFF_START	12

#else	/* ! __ASSEMBLY__ */

typedef struct {
	ulong	size;			/* total bank size in bytes		*/
	ushort	sector_count;		/* number of erase units		*/
	ushort	flash_id;		/* combined device & manufacturer code	*/
	ulong	start[CFG_MAX_FLASH_SECT];   /* physical sector start addresses	*/
	uchar	protect[CFG_MAX_FLASH_SECT]; /* sector protection status	*/
} flash_info_t;

/* Prototypes */

unsigned long flash_init (void);
void	flash_print_info (flash_info_t *);
void	flash_erase	 (flash_info_t *, int, int);

#endif	/* __ASSEMBLY__ */

/*-----------------------------------------------------------------------
 * Device IDs for AMD and Fujitsu FLASH
 */

#define AMD_MANUFACT	0x00010001	/* AMD     manuf. ID in D23..D16, D7..D0 */
#define FUJ_MANUFACT	0x00040004	/* FUJITSU manuf. ID in D23..D16, D7..D0 */
#define STM_MANUFACT	0x00200020	/* STM (Thomson) manuf. ID in D23.. -"-	*/
#define SST_MANUFACT	0x00BF00BF	/* SST     manuf. ID in D23..D16, D7..D0 */

#define AMD_ID_F040B	0xA4		/* 29F040B ID  ( 4 M, bottom boot sect)	*/

#define AMD_ID_LV400T	0x22B922B9	/* 29LV400T ID ( 4 M, top boot sector)	*/
#define AMD_ID_LV400B	0x22BA22BA	/* 29LV400B ID ( 4 M, bottom boot sect)	*/

#define AMD_ID_LV800T	0x22DA22DA	/* 29LV800T ID ( 8 M, top boot sector)	*/
#define AMD_ID_LV800B	0x225B225B	/* 29LV800B ID ( 8 M, bottom boot sect)	*/

#define AMD_ID_LV160T	0x22C422C4	/* 29LV160T ID (16 M, top boot sector)	*/
#define AMD_ID_LV160B	0x22492249	/* 29LV160B ID (16 M, bottom boot sect)	*/

/* 29LV320 device IDs are not yet available */
#define AMD_ID_LV320T	0xDEADBEEF	/* 29LV320T ID (32 M, top boot sector)	*/
#define AMD_ID_LV320B	0xDEADBEEF	/* 29LV320B ID (32 M, bottom boot sect)	*/

#define SST_ID_xF200A	0x27892789	/* 39xF200A ID ( 2M = 128K x 16	)	*/
#define SST_ID_xF400A	0x27802780	/* 39xF400A ID ( 4M = 256K x 16	)	*/
#define SST_ID_xF800A	0x27812781	/* 39xF800A ID ( 8M = 512K x 16	)	*/
#define SST_ID_xF160A	0x27822782	/* 39xF800A ID (16M =   1M x 16 )	*/

#define STM_ID_x800AB	0x005B005B	/* M29W800AB ID (8M = 512K x 16	)	*/

/*-----------------------------------------------------------------------
 * Internal FLASH identification codes
 */

#define FLASH_AM040T	0x00		/* AMD AM29F040			*/
#define FLASH_AM040B	0x01
#define FLASH_AM400T	0x02		/* AMD AM29LV400			*/
#define FLASH_AM400B	0x03
#define FLASH_AM800T	0x04		/* AMD AM29LV800			*/
#define FLASH_AM800B	0x05
#define FLASH_AM160T	0x06		/* AMD AM29LV160			*/
#define FLASH_AM160B	0x07
#define FLASH_AM320T	0x08		/* AMD AM29LV320			*/
#define FLASH_AM320B	0x09

#define	FLASH_SST200A	0x0A		/* SST 39xF200A ID ( 2M = 128K x 16 )	*/
#define	FLASH_SST400A	0x0B		/* SST 39xF400A ID ( 4M = 256K x 16 )	*/
#define	FLASH_SST800A	0x0C		/* SST 39xF800A ID ( 8M = 512K x 16 )	*/
#define FLASH_SST160A	0x0D		/* SST 39xF160A ID (16M =   1M x 16 )	*/

#define	FLASH_STM800AB	0x0F		/* STM M29WF800AB ID ( 8M = 512K x 16 )	*/

#define FLASH_UNKNOWN	0xFF		/* unknown flash type			*/


#define FLASH_MAN_AMD	0x00		/* manufacturer offsets			*/
#define FLASH_MAN_FUJ	0x10
#define FLASH_MAN_SST	0x20
#define FLASH_MAN_STM	0x40


#define FLASH_TYPEMASK	0x0F		/* extract FLASH type   information	*/
#define FLASH_VENDMASK	0xF0		/* extract FLASH vendor information	*/

#define FLASH_AMD_COMP	0x4F		/* Up to this ID, FLASH is compatible	*/
					/* with AMD, Fujitsu and SST		*/
					/* (JEDEC standard commands ?)		*/

#define FLASH_BTYPE	0x01		/* mask for bottom boot sector type	*/

/*-----------------------------------------------------------------------
 * Timeout constants:
 *
 * We can't find any spcifications for maximum chip erase times,
 * so these values are guestimates.
 */
#define FLASH_ERASE_TIMEOUT	120000	/* timeout for erasing in ms		*/
#define FLASH_WRITE_TIMEOUT	500	/* timeout for writes  in ms		*/

